{{
Microchip MCP23008 8-bit I/O Expander I2C Object
by Clive Wakeham

This chip has three levels of I2C speed 100kHz, 400kHz and 1.7MHz
It also has three address pins allowing it to be configured to 8 different I2C addresses or allow
up to 8 different MCP23008's to be interfaced on the same I2C bus.

Configurable interrupt_out pin which can be Active-High, Active-Low or Open-Drain.
Configurable interrupt source -- interrupt on change from default value (settable) or pin polarity change.
Polarity inversion register to configure the polarity of the input port pins data.

This device allows a programmer/designer to have multiple IC's that can be connected to a Prop without
utilizing alot of Prop pins. This allows up to eight other devices to be connected to the Prop and use only
one pin (other than the SDA/SCL pins). The other IC's can flag the Prop via the MCP23008 chip, and the MCP23008
chip can flag the Prop via its INT output.

Version 1.0  3rd April 2011 -- Original
Version 1.1  4th April 2011 -- Changed calling method.
Version 1.2 22nd April 2011 -- Updated calling schemes.
Version 1.3    6th May 2011 -- Updated documentation.

                            AAA
                            210
'base device address = %0100XXX0                                                ' 40 hex write -- 41 hex read to 4E hex write -- 4F hex read
}}

obj
                I2C             : "Basic_I2C_Driver"                            'Basic I2C driver in SPIN

con
                  device_address = %01000000                                    'Base device address
                  SCL            = 28                                           'Base address for SCL pin. SDA = SCL + 1
                  
var
                     byte       register_address
                                                            
pub Read_Port | port_data
{{Reads the actual value on the port. Returns an 8 bit value of current pin values. Including those set as outputs as well as inputs.}}
register_address := $09                                                         
I2C.ReadLocation(SCL, device_address, register_address)                             
return result

pub Write_Port(data)
{{Writes to the GPIO register and OLAT (Output latches). Doesn't effect pins configured as inputs. }}
register_address := $09                                                        
I2C.WriteLocation(SCL, device_address, register_address, data)
return

pub Read_OLATCH | output_latch_data
{{Reads the OLAT register NOT the actual value on the port. Read doumentation for the write_olatch method }}
register_address := $0A                                                         
I2C.ReadLocation(SCL, device_address, register_address)  
return result

pub Write_OLATCH(data)
{{Writes to the OLAT register. This forces the associated output drivers to drive to the level in OLAT.
Writing to pins configured as inputs will not change those pins.
Pins configured as inputs turn off the associated output driver and put it in a high-impedance state.}}
register_address := $0A                                                              
I2C.WriteLocation(SCL, device_address, register_address, data)
return

pub Read_IODIR | port_direction_data
{{Read the IODIR register to determine how the pins are set up for I/O Direction register. }}
register_address := $00
I2C.ReadLocation(SCL, device_address, register_address)  
return result

pub Write_IODIR(data)
{{Write to IODIR register. (I/O Direction register)
If the bit is cleared then pin is output, bit is set then pin is input. Which is opposite to how the Propeller assigns its pins!}}     
register_address := $00                                                         
I2C.WriteLocation(SCL, device_address, register_address, data)
return

pub Read_IPOL | port_polarity_data
{{Read the IPOL register to determine how the pins are set up for Input Polarity register. }}
register_address := $01
I2C.ReadLocation(SCL, device_address, register_address)  
return result

pub Write_IPOL(data)
{{Write to IPOL register. (Input Polarity register)
If the bit is cleared then the corresponding GPIO register bit is true to the input
If the bit is set then the corresponding GPIO register bit is opposite to the input }}     
register_address := $01                                                          
I2C.WriteLocation(SCL, device_address, register_address, data)                       
return

pub Read_GPINTEN | interrupt_on_change
{{Read the GPINTEN register to determine how the pins are set up for Interrupt-on-change. }}
register_address := $02
I2C.ReadLocation(SCL, device_address, register_address)  
return result

pub Write_GPINTEN(data)
{{Write to the GPINTEN register. (Interrupt-on-change register)
If bit is set then the corresponding pin is enabled for interrupt-on-change.
The DEFVAL and INTCON registers must also be configured if any pins are enabled for interrupt-on-change.}} 
register_address := $02                                                          
I2C.WriteLocation(SCL, device_address, register_address, data)                        
return

pub Read_DEFVAL | default_value_data
{{Read the DEFVAL register to determine how the pins are set up for Default Value comparison.}}
register_address := $03
I2C.ReadLocation(SCL, device_address, register_address)  
return result

pub Write_DEFVAL(data)
{{Write to the DEFVAL register. (Default Compare register)
Default comparison value. If enabled via the GPINTEN and INTCON registers.
Any opposite value on the associated pin will cause an interrupt to occur.
So any pin set up high and then goes low will trigger an interupt, and vis-versa.}} 
register_address := $03                                                          
I2C.WriteLocation(SCL, device_address, register_address, data)                        
return                                                                               

pub Read_INTCON | interrupt_condition
{{Read the INTCON register to determine how interrupts are set up for the pins. }}
register_address := $04
I2C.ReadLocation(SCL, device_address, register_address)  
return result

pub Write_INTCON(data)
{{Write to the INTCON register. (Interrupt Control register)
This controls how the associated pin value is compared for the interrupt-on-change feature.
If the bit is set then it is compared to DEFVAL register,
if the bit is clear then it is compared to its previous value.}} 
register_address := $04                                                          
I2C.WriteLocation(SCL, device_address, register_address, data)                       
return                                                                          

pub Read_IOCON | IOCON_data
{{Read the IOCON register to determine SEQOP, DISSLW, ODR and INTPOL bits are configured. }}
register_address := $05
I2C.ReadLocation(SCL, device_address, register_address)  
return result

pub Write_IOCON(data)
{{Write to the IOCON register. (Configuration register)
Bits 7, 6, 3 and 0 are not implemented and writing to those bits are ignored.
Bit 5 (SEQOP) Sequential Operation Mode bit -- Set = Sequential operation disabled -- register address pointer does not increment.
                                             Clear = Sequential operation enabled -- register address pointer increments.
Bit 4 (DISSLW) Slew Rate control bit for SDA.  Set = Slew Rate disabled. Clear = Slew Rate enabled.
Bit 2 (ODR) INT pin control. Set = Open drain output (overrides the INTPOL bit). Clear = Active driver output (INTPOL bit sets the polarity)
Bit 1 (INTPOL) INT Polarity control. Set = Active High. Clear = Active Low  }}
register_address := $05
I2C.WriteLocation(SCL, device_address, register_address, data)
return 

pub Read_GPPU | pull_resistor_data
{{Read the GPPU register to determine how the pull-up resistors are configured.}}
register_address := $06
I2C.ReadLocation(SCL, device_address, register_address)  
return result

pub Write_GPPU(data)
{{Writes to the GPPU register (Pull-up resistor Configuration register)
If a bit is set and the corresponding pin is configured as a input, the corresponding port pin is internally pulled up with a 100K resistor
If bit is clear or pin is an output then resistor is disabled. }}
register_address := $06
I2C.WriteLocation(SCL, device_address, register_address, data)
return 

pub Read_INTF | interrupt_condition_data2
{{The INTF register reflects the interrupt condition on the port pins of any pin that is enabled for interrupts via the GPINTEN register.
a "set" bit indicates that the associated pin caused the interrupt.
THE INTF register will always reflect the pin(s) that caused the interrupt. Even if another pin changed after the original interrupt happened. }}
register_address := $07
I2C.ReadLocation(SCL, device_address, register_address)  
return result

pub Read_INTCAP | interrupt_condition_data
{{The INTCAP register captures the GPIO port value at the time the interrupt occurred.
The register will remain unchanged until the interrupt is cleared via a read of INTCAP or GPIO
A "set" bit indicates that the associated pin caused the interrupt. Captures the original interrupt. }}
register_address := $08
I2C.ReadLocation(SCL, device_address, register_address)  
return result

{{

  
}}

{{
********************************************************************************************************************************
*                                                   TERMS OF USE: MIT License                                                  *                                                            
********************************************************************************************************************************
*Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation    * 
*files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy,    *
*modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software*
*is furnished to do so, subject to the following conditions:                                                                   *
*                                                                                                                              *
*The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.*
*                                                                                                                              *
*THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE          *
*WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR         *
*COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,   *
*ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.                         *
********************************************************************************************************************************
}}
                                                                                                                                     